@@ -16,14 +16,16 @@ mod bridge_name_tracker;
 pub(crate) mod function_wrapper;
 mod overload_tracker;
 mod rust_name_tracker;
+mod subclass;
 
 use crate::{
     conversion::{
         analysis::{
+            fun::function_wrapper::CppFunctionKind,
             has_attr,
             type_converter::{add_analysis, TypeConversionContext, TypeConverter},
         },
-        api::{ApiName, FuncToConvert},
+        api::{ApiName, FuncToConvert, SubclassName},
         convert_error::ConvertErrorWithContext,
         convert_error::ErrorContext,
         error_reporter::convert_apis,
@@ -34,25 +36,27 @@ use crate::{
 use std::collections::{HashMap, HashSet};
 
 use autocxx_parser::{IncludeCppConfig, UnsafePolicy};
-use function_wrapper::{FunctionWrapper, FunctionWrapperPayload, TypeConversionPolicy};
+use function_wrapper::{CppFunction, CppFunctionBody, TypeConversionPolicy};
+use itertools::Itertools;
 use proc_macro2::Span;
 use syn::{
-    parse_quote, punctuated::Punctuated, FnArg, ForeignItemFn, Ident, LitStr, Pat, ReturnType,
-    Type, TypePtr, Visibility,
+    parse_quote, punctuated::Punctuated, token::Comma, FnArg, ForeignItemFn, Ident, LitStr, Pat,
+    ReturnType, Type, TypePtr, Visibility,
 };
 
 use crate::{
     conversion::{
         api::{AnalysisPhase, Api, TypeKind, UnanalyzedApi},
-        codegen_cpp::AdditionalNeed,
         ConvertError,
     },
     types::{make_ident, validate_ident_ok_for_cxx, Namespace, QualifiedName},
 };
 
 use self::{
-    bridge_name_tracker::BridgeNameTracker, overload_tracker::OverloadTracker,
+    bridge_name_tracker::BridgeNameTracker,
+    overload_tracker::OverloadTracker,
     rust_name_tracker::RustNameTracker,
+    subclass::{add_subclass_constructor, create_subclass_function},
 };
 
 use super::{
@@ -60,14 +64,22 @@ use super::{
     tdef::TypedefAnalysisBody,
 };
 
-pub(crate) enum MethodKind {
-    Normal,
-    Constructor,
-    Static,
-    Virtual,
-    PureVirtual,
+#[derive(Clone, Debug)]
+pub(crate) enum ReceiverMutability {
+    Const,
+    Mutable,
 }
 
+#[derive(Clone)]
+pub(crate) enum MethodKind {
+    Normal(ReceiverMutability),
+    Constructor,
+    Static,
+    Virtual(ReceiverMutability),
+    PureVirtual(ReceiverMutability),
+}
+
+#[derive(Clone)]
 pub(crate) enum FnKind {
     Function,
     Method(QualifiedName, MethodKind),
@@ -75,6 +87,8 @@ pub(crate) enum FnKind {
 
 /// Strategy for ensuring that the final, callable, Rust name
 /// is what the user originally expected.
+#[derive(Clone)]
+
 pub(crate) enum RustRenameStrategy {
     /// cxx::bridge name matches user expectations
     None,
@@ -85,28 +99,31 @@ pub(crate) enum RustRenameStrategy {
     RenameInOutputMod(Ident),
 }
 
+#[derive(Clone)]
 pub(crate) struct FnAnalysisBody {
     pub(crate) cxxbridge_name: Ident,
     pub(crate) rust_name: String,
     pub(crate) rust_rename_strategy: RustRenameStrategy,
-    pub(crate) params: Punctuated<FnArg, syn::Token![,]>,
+    pub(crate) params: Punctuated<FnArg, Comma>,
     pub(crate) kind: FnKind,
     pub(crate) ret_type: ReturnType,
     pub(crate) param_details: Vec<ArgumentAnalysis>,
+    pub(crate) ret_conversion: Option<TypeConversionPolicy>,
     pub(crate) requires_unsafe: bool,
     pub(crate) vis: Visibility,
-    pub(crate) cpp_wrapper: Option<AdditionalNeed>,
+    pub(crate) cpp_wrapper: Option<CppFunction>,
     pub(crate) deps: HashSet<QualifiedName>,
 }
 
+#[derive(Clone)]
 pub(crate) struct ArgumentAnalysis {
     pub(crate) conversion: TypeConversionPolicy,
     pub(crate) name: Pat,
-    pub(crate) self_type: Option<QualifiedName>,
-    was_reference: bool,
-    deps: HashSet<QualifiedName>,
-    is_virtual: bool,
-    requires_unsafe: bool,
+    pub(crate) self_type: Option<(QualifiedName, ReceiverMutability)>,
+    pub(crate) was_reference: bool,
+    pub(crate) deps: HashSet<QualifiedName>,
+    pub(crate) is_virtual: bool,
+    pub(crate) requires_unsafe: bool,
 }
 
 struct ReturnTypeAnalysis {
@@ -133,7 +150,7 @@ pub(crate) struct FnAnalyzer<'a> {
     pod_safe_types: HashSet<QualifiedName>,
     config: &'a IncludeCppConfig,
     overload_trackers_by_mod: HashMap<Namespace, OverloadTracker>,
-    generate_utilities: bool,
+    subclasses_by_superclass: HashMap<QualifiedName, Vec<SubclassName>>,
 }
 
 impl<'a> FnAnalyzer<'a> {
@@ -151,13 +168,13 @@ impl<'a> FnAnalyzer<'a> {
             config,
             overload_trackers_by_mod: HashMap::new(),
             pod_safe_types: Self::build_pod_safe_type_set(&apis),
-            generate_utilities: Self::should_generate_utilities(&apis),
+            subclasses_by_superclass: subclass::subclasses_by_superclass(&apis),
         };
         let mut results = Vec::new();
         convert_apis(
             apis,
             &mut results,
-            |name, fun, _| me.analyze_foreign_fn(name, fun),
+            |name, fun, _, _| me.analyze_foreign_fn_and_subclasses(name, fun),
             Api::struct_unchanged,
             Api::enum_unchanged,
             Api::typedef_unchanged,
@@ -166,11 +183,6 @@ impl<'a> FnAnalyzer<'a> {
         results
     }
 
-    fn should_generate_utilities(apis: &[Api<PodAnalysis>]) -> bool {
-        apis.iter()
-            .any(|api| matches!(api, Api::StringConstructor { .. }))
-    }
-
     fn build_pod_safe_type_set(apis: &[Api<PodAnalysis>]) -> HashSet<QualifiedName> {
         apis.iter()
             .filter_map(|api| match api {
@@ -244,6 +256,79 @@ impl<'a> FnAnalyzer<'a> {
         self.unsafe_policy == UnsafePolicy::AllFunctionsUnsafe
     }
 
+    /// Analyze a given function, and consider if the APIs that we
+    /// find also need to be replicated in subclasses.
+    fn analyze_foreign_fn_and_subclasses(
+        &mut self,
+        name: ApiName,
+        func_information: Box<FuncToConvert>,
+    ) -> Result<Box<dyn Iterator<Item = Api<FnAnalysis>>>, ConvertErrorWithContext> {
+        let maybe_analysis_and_name = self.analyze_foreign_fn(name, func_information.clone())?;
+
+        let (analysis, name) = match maybe_analysis_and_name {
+            None => return Ok(Box::new(std::iter::empty())),
+            Some((analysis, name)) => (analysis, name),
+        };
+
+        let mut results = Vec::new();
+
+        // Consider whether we need to synthesize subclass items.
+        match &analysis.kind {
+            FnKind::Method(sup, MethodKind::Constructor) => {
+                for sub in self.subclasses_by_superclass(sup) {
+                    if analysis.param_details.len() > 0 {
+                        return Err(ConvertErrorWithContext(
+                            ConvertError::SuperclassConstructorsTooFancy(sup.clone()),
+                            None,
+                        ));
+                    }
+                    add_subclass_constructor(&sub, &mut results, &analysis, &func_information);
+                }
+            }
+            FnKind::Method(
+                sup,
+                MethodKind::Virtual(receiver_mutability)
+                | MethodKind::PureVirtual(receiver_mutability),
+            ) => {
+                for sub in self.subclasses_by_superclass(sup) {
+                    let subclass_function =
+                        create_subclass_function(&sub, &analysis, &name, receiver_mutability, sup);
+                    results.push(subclass_function);
+
+                    let mut item = func_information.item.clone();
+                    item.sig.ident = SubclassName::get_super_fn_name(&analysis.rust_name);
+                    let self_ty = Some(QualifiedName::new(&Namespace::new(), sub.cpp()));
+                    let maybe_wrap = Box::new(FuncToConvert {
+                        item,
+                        virtual_this_type: self_ty.clone(),
+                        self_ty,
+                    });
+                    let name =
+                        ApiName::new(name.name.get_namespace(), maybe_wrap.item.sig.ident.clone());
+                    let maybe_another_api = self.analyze_foreign_fn(name, maybe_wrap)?;
+                    if let Some((analysis, name)) = maybe_another_api {
+                        results.push(Api::Function {
+                            fun: func_information.clone(),
+                            analysis,
+                            name,
+                            name_for_gc: None,
+                        });
+                    }
+                }
+            }
+            _ => {}
+        }
+
+        results.push(Api::Function {
+            fun: func_information,
+            analysis,
+            name,
+            name_for_gc: None,
+        });
+
+        Ok(Box::new(results.into_iter()))
+    }
+
     /// Determine how to materialize a function.
     ///
     /// The main job here is to determine whether a function can simply be noted
@@ -263,7 +348,7 @@ impl<'a> FnAnalyzer<'a> {
         &mut self,
         name: ApiName,
         func_information: Box<FuncToConvert>,
-    ) -> Result<Option<Api<FnAnalysis>>, ConvertErrorWithContext> {
+    ) -> Result<Option<(FnAnalysisBody, ApiName)>, ConvertErrorWithContext> {
         let fun = &func_information.item;
         let virtual_this = &func_information.virtual_this_type;
         let mut cpp_name = name.cpp_name.clone();
@@ -281,7 +366,7 @@ impl<'a> FnAnalyzer<'a> {
 
         // Now let's analyze all the parameters.
         // See if any have annotations which our fork of bindgen has craftily inserted...
-        let (reference_params, reference_return) = Self::get_reference_parameters_and_return(&fun);
+        let (reference_params, reference_return) = Self::get_reference_parameters_and_return(fun);
         let (param_details, bads): (Vec<_>, Vec<_>) = fun
             .sig
             .inputs
@@ -289,14 +374,14 @@ impl<'a> FnAnalyzer<'a> {
             .map(|i| {
                 self.convert_fn_arg(
                     i,
-                    &ns,
+                    ns,
                     diagnostic_display_name,
                     virtual_this.clone(),
                     &reference_params,
                 )
             })
             .partition(Result::is_ok);
-        let (mut params, mut param_details): (Punctuated<_, syn::Token![,]>, Vec<_>) =
+        let (mut params, mut param_details): (Punctuated<_, Comma>, Vec<_>) =
             param_details.into_iter().map(Result::unwrap).unzip();
 
         let params_deps: HashSet<_> = param_details
@@ -332,7 +417,7 @@ impl<'a> FnAnalyzer<'a> {
             Some(cpp_name) => {
                 if initial_rust_name.ends_with('_') {
                     initial_rust_name // case 2
-                } else if validate_ident_ok_for_rust(&cpp_name).is_err() {
+                } else if validate_ident_ok_for_rust(cpp_name).is_err() {
                     format!("{}_", cpp_name) // case 5
                 } else {
                     cpp_name.to_string() // cases 3, 4, 6
@@ -342,13 +427,16 @@ impl<'a> FnAnalyzer<'a> {
 
         // Let's spend some time figuring out the kind of this function (i.e. method,
         // virtual function, etc.)
-        let (is_static_method, self_ty) = if self_ty.is_none() {
-            // Even if we can't find a 'self' parameter this could conceivably
-            // be a static method.
-            let self_ty = func_information.self_ty.clone();
-            (self_ty.is_some(), self_ty)
-        } else {
-            (false, self_ty)
+        let (is_static_method, self_ty, receiver_mutability) = match self_ty {
+            None => {
+                // Even if we can't find a 'self' parameter this could conceivably
+                // be a static method.
+                let self_ty = func_information.self_ty.clone();
+                (self_ty.is_some(), self_ty, None)
+            }
+            Some((self_ty, receiver_mutability)) => {
+                (false, Some(self_ty), Some(receiver_mutability))
+            }
         };
 
         let (kind, error_context, rust_name) = if let Some(self_ty) = self_ty {
@@ -370,7 +458,7 @@ impl<'a> FnAnalyzer<'a> {
             // We want to feed cxx methods with just the method name, so let's
             // strip off the class name.
             let overload_tracker = self.overload_trackers_by_mod.entry(ns.clone()).or_default();
-            let mut rust_name = overload_tracker.get_method_real_name(&type_ident, ideal_rust_name);
+            let mut rust_name = overload_tracker.get_method_real_name(type_ident, ideal_rust_name);
             let method_kind = if rust_name.starts_with(&type_ident) {
                 // It's a constructor. bindgen generates
                 // fn new(this: *Type, ...args)
@@ -388,14 +476,18 @@ impl<'a> FnAnalyzer<'a> {
                 MethodKind::Constructor
             } else if is_static_method {
                 MethodKind::Static
-            } else if param_details.iter().any(|pd| pd.is_virtual) {
-                if has_attr(&fun.attrs, "bindgen_pure_virtual") {
-                    MethodKind::PureVirtual
-                } else {
-                    MethodKind::Virtual
-                }
             } else {
-                MethodKind::Normal
+                let receiver_mutability =
+                    receiver_mutability.expect("Failed to find receiver details");
+                if param_details.iter().any(|pd| pd.is_virtual) {
+                    if has_attr(&fun.attrs, "bindgen_pure_virtual") {
+                        MethodKind::PureVirtual(receiver_mutability)
+                    } else {
+                        MethodKind::Virtual(receiver_mutability)
+                    }
+                } else {
+                    MethodKind::Normal(receiver_mutability)
+                }
             };
             let error_context = ErrorContext::Method {
                 self_ty: self_ty.get_final_ident(),
@@ -427,7 +519,7 @@ impl<'a> FnAnalyzer<'a> {
                 FnKind::Function => None,
             },
             &rust_name,
-            &ns,
+            ns,
         );
         if cxxbridge_name != rust_name && cpp_name.is_none() {
             cpp_name = Some(rust_name.clone());
@@ -454,7 +546,7 @@ impl<'a> FnAnalyzer<'a> {
         }
 
         // Reject move constructors.
-        if Self::is_move_constructor(&fun) {
+        if Self::is_move_constructor(fun) {
             return Err(contextualize_error(
                 ConvertError::MoveConstructorUnsupported,
             ));
@@ -463,7 +555,7 @@ impl<'a> FnAnalyzer<'a> {
         match kind {
             FnKind::Method(_, MethodKind::Static) => {}
             FnKind::Method(ref self_ty, _) => {
-                if !known_types().is_cxx_acceptable_receiver(&self_ty) {
+                if !known_types().is_cxx_acceptable_receiver(self_ty) {
                     return Err(contextualize_error(ConvertError::UnsupportedReceiver));
                 }
             }
@@ -475,8 +567,6 @@ impl<'a> FnAnalyzer<'a> {
         let mut return_analysis = if let FnKind::Method(ref self_ty, MethodKind::Constructor) = kind
         {
             let constructed_type = self_ty.to_type_path();
-            let mut these_deps = HashSet::new();
-            these_deps.insert(self_ty.clone());
             ReturnTypeAnalysis {
                 rt: parse_quote! {
                     -> #constructed_type
@@ -485,10 +575,10 @@ impl<'a> FnAnalyzer<'a> {
                     #constructed_type
                 })),
                 was_reference: false,
-                deps: these_deps,
+                deps: std::iter::once(self_ty).cloned().collect(),
             }
         } else {
-            self.convert_return_type(&fun.sig.output, &ns, reference_return)
+            self.convert_return_type(&fun.sig.output, ns, reference_return)
                 .map_err(contextualize_error)?
         };
         let mut deps = params_deps;
@@ -515,7 +605,7 @@ impl<'a> FnAnalyzer<'a> {
         // See https://github.com/dtolnay/cxx/issues/878 for the reason for this next line.
         let effective_cpp_name = cpp_name.as_ref().unwrap_or(&rust_name);
         let cpp_name_incompatible_with_cxx =
-            validate_ident_ok_for_rust(&effective_cpp_name).is_err();
+            validate_ident_ok_for_rust(effective_cpp_name).is_err();
         // If possible, we'll put knowledge of the C++ API directly into the cxx::bridge
         // mod. However, there are various circumstances where cxx can't work with the existing
         // C++ API and we need to create a C++ wrapper function which is more cxx-compliant.
@@ -523,8 +613,8 @@ impl<'a> FnAnalyzer<'a> {
         // original function.
         let wrapper_function_needed = match kind {
             FnKind::Method(_, MethodKind::Static)
-            | FnKind::Method(_, MethodKind::Virtual)
-            | FnKind::Method(_, MethodKind::PureVirtual) => true,
+            | FnKind::Method(_, MethodKind::Virtual(_))
+            | FnKind::Method(_, MethodKind::PureVirtual(_)) => true,
             FnKind::Method(..) if cxxbridge_name != rust_name => true,
             _ if param_conversion_needed => true,
             _ if ret_type_conversion_needed => true,
@@ -543,11 +633,9 @@ impl<'a> FnAnalyzer<'a> {
             };
             cxxbridge_name = make_ident(&format!("{}{}autocxx_wrapper", cxxbridge_name, joiner));
             let (payload, has_receiver) = match kind {
-                FnKind::Method(_, MethodKind::Constructor) => {
-                    (FunctionWrapperPayload::Constructor, false)
-                }
+                FnKind::Method(_, MethodKind::Constructor) => (CppFunctionBody::Constructor, false),
                 FnKind::Method(ref self_ty, MethodKind::Static) => (
-                    FunctionWrapperPayload::StaticMethodCall(
+                    CppFunctionBody::StaticMethodCall(
                         ns.clone(),
                         self_ty.get_final_ident(),
                         cpp_construction_ident,
@@ -555,11 +643,11 @@ impl<'a> FnAnalyzer<'a> {
                     false,
                 ),
                 FnKind::Method(..) => (
-                    FunctionWrapperPayload::FunctionCall(ns.clone(), cpp_construction_ident),
+                    CppFunctionBody::FunctionCall(ns.clone(), cpp_construction_ident),
                     true,
                 ),
                 _ => (
-                    FunctionWrapperPayload::FunctionCall(ns.clone(), cpp_construction_ident),
+                    CppFunctionBody::FunctionCall(ns.clone(), cpp_construction_ident),
                     false,
                 ),
             };
@@ -587,13 +675,19 @@ impl<'a> FnAnalyzer<'a> {
                 ));
             }
 
-            Some(AdditionalNeed::FunctionWrapper(Box::new(FunctionWrapper {
+            Some(CppFunction {
                 payload,
                 wrapper_function_name: cxxbridge_name.clone(),
-                return_conversion: ret_type_conversion,
+                return_conversion: ret_type_conversion.clone(),
                 argument_conversion: param_details.iter().map(|d| d.conversion.clone()).collect(),
-                is_a_method: has_receiver,
-            })))
+                kind: if has_receiver {
+                    CppFunctionKind::Method
+                } else {
+                    CppFunctionKind::Function
+                },
+                pass_obs_field: false,
+                qualification: None,
+            })
         } else {
             None
         };
@@ -625,26 +719,32 @@ impl<'a> FnAnalyzer<'a> {
             }
         };
 
-        Ok(Some(Api::Function {
-            fun: func_information,
-            analysis: FnAnalysisBody {
-                cxxbridge_name,
-                rust_name,
-                rust_rename_strategy,
-                params,
-                kind,
-                ret_type,
-                param_details,
-                requires_unsafe,
-                vis,
-                cpp_wrapper,
-                deps,
-            },
-            name: ApiName {
-                cpp_name,
-                name: QualifiedName::new(ns, id),
-            },
-        }))
+        let analysis = FnAnalysisBody {
+            cxxbridge_name,
+            rust_name: rust_name.clone(),
+            rust_rename_strategy,
+            params,
+            ret_conversion: ret_type_conversion,
+            kind,
+            ret_type,
+            param_details,
+            requires_unsafe,
+            vis,
+            cpp_wrapper,
+            deps,
+        };
+        let name = ApiName {
+            cpp_name,
+            name: QualifiedName::new(ns, id),
+        };
+        Ok(Some((analysis, name)))
+    }
+
+    fn subclasses_by_superclass(&self, sup: &QualifiedName) -> impl Iterator<Item = SubclassName> {
+        match self.subclasses_by_superclass.get(sup) {
+            Some(subs) => subs.clone().into_iter(),
+            None => Vec::new().into_iter(),
+        }
     }
 
     fn convert_fn_arg(
@@ -669,6 +769,11 @@ impl<'a> FnAnalyzer<'a> {
                                 elem, mutability, ..
                             }) => match elem.as_ref() {
                                 Type::Path(typ) => {
+                                    let receiver_mutability = if mutability.is_some() {
+                                        ReceiverMutability::Mutable
+                                    } else {
+                                        ReceiverMutability::Const
+                                    };
                                     let mut this_type = QualifiedName::from_type_path(typ);
                                     if this_type.is_cvoid() && pp.ident == "this" {
                                         is_virtual = true;
@@ -688,7 +793,7 @@ impl<'a> FnAnalyzer<'a> {
                                             * #mutability #const_token #this_type_path
                                         });
                                     }
-                                    Ok(this_type)
+                                    Ok((this_type, receiver_mutability))
                                 }
                                 _ => Err(ConvertError::UnexpectedThisType(
                                     ns.clone(),
@@ -738,7 +843,9 @@ impl<'a> FnAnalyzer<'a> {
                 let tn = QualifiedName::from_type_path(p);
                 if self.pod_safe_types.contains(&tn) {
                     TypeConversionPolicy::new_unconverted(ty.clone())
-                } else if known_types().convertible_from_strs(&tn) && self.generate_utilities {
+                } else if known_types().convertible_from_strs(&tn)
+                    && !self.config.exclude_utilities()
+                {
                     TypeConversionPolicy::new_from_str(ty.clone())
                 } else {
                     TypeConversionPolicy::new_from_unique_ptr(ty.clone())
@@ -833,42 +940,48 @@ impl<'a> FnAnalyzer<'a> {
 impl Api<FnAnalysis> {
     pub(crate) fn typename_for_allowlist(&self) -> QualifiedName {
         match &self {
+            Api::Function {
+                name_for_gc: Some(name),
+                ..
+            } => name.clone(),
             Api::Function { analysis, .. } => match analysis.kind {
                 FnKind::Method(ref self_ty, _) => self_ty.clone(),
-                FnKind::Function => QualifiedName::new(
-                    &self.name().get_namespace(),
-                    make_ident(&analysis.rust_name),
-                ),
+                FnKind::Function => {
+                    QualifiedName::new(self.name().get_namespace(), make_ident(&analysis.rust_name))
+                }
             },
+            Api::RustSubclassFn { subclass, .. }
+            | Api::RustSubclassConstructor { subclass, .. } => subclass.0.name.clone(),
             _ => self.name().clone(),
         }
     }
 
-    /// Whether this API requires generation of additional C++, and if so,
-    /// what.
-    /// This seems an odd place for this function (as opposed to in the [codegen_rs]
+    /// Whether this API requires generation of additional C++.
+    /// This seems an odd place for this function (as opposed to in the [codegen_cpp]
     /// module) but, as it happens, even our Rust codegen phase needs to know if
     /// more C++ is needed (so it can add #includes in the cxx mod).
     /// And we can't answer the question _prior_ to this function analysis phase.
-    pub(crate) fn additional_cpp(&self) -> Option<AdditionalNeed> {
+    pub(crate) fn needs_cpp_codegen(&self) -> bool {
         match &self {
-            Api::Function { analysis, .. } => analysis.cpp_wrapper.clone(),
-            Api::StringConstructor { .. } => Some(AdditionalNeed::MakeStringConstructor),
-            Api::ConcreteType { rs_definition, .. } => {
-                Some(AdditionalNeed::ConcreteTemplatedTypeTypedef(
-                    self.name().clone(),
-                    rs_definition.clone(),
-                ))
-            }
-            Api::CType { typename, .. } => Some(AdditionalNeed::CTypeTypedef(typename.clone())),
-            _ => None,
+            Api::Function { analysis, .. } => analysis.cpp_wrapper.is_some(),
+            Api::StringConstructor { .. }
+            | Api::ConcreteType { .. }
+            | Api::CType { .. }
+            | Api::RustSubclassConstructor { .. }
+            | Api::RustSubclassFn { .. }
+            | Api::Subclass { .. } => true,
+            _ => false,
         }
     }
 
     pub(crate) fn cxxbridge_name(&self) -> Option<Ident> {
         match self {
             Api::Function { ref analysis, .. } => Some(analysis.cxxbridge_name.clone()),
-            Api::StringConstructor { .. } | Api::Const { .. } | Api::IgnoredItem { .. } => None,
+            Api::StringConstructor { .. }
+            | Api::Const { .. }
+            | Api::IgnoredItem { .. }
+            | Api::RustSubclassConstructor { .. }
+            | Api::RustSubclassFn { .. } => None,
             _ => Some(self.name().get_final_ident()),
         }
     }
@@ -883,7 +996,18 @@ impl Api<FnAnalysis> {
             } => Box::new(old_tyname.iter().chain(deps.iter())),
             Api::Struct { analysis, .. } => Box::new(analysis.field_deps.iter()),
             Api::Function { analysis, .. } => Box::new(analysis.deps.iter()),
+            Api::Subclass {
+                name: _,
+                superclass,
+            } => Box::new(std::iter::once(superclass)),
+            Api::RustSubclassFn { details, .. } => {
+                Box::new(std::iter::once(&details.super_fn_api_name))
+            }
             _ => Box::new(std::iter::empty()),
         }
     }
+
+    pub(crate) fn format_deps(&self) -> String {
+        self.deps().join(",")
+    }
 }
