@@ -17,7 +17,11 @@ use std::collections::HashMap;
 use syn::Ident;
 
 use crate::{
-    conversion::{api::Api, error_reporter::convert_item_apis, ConvertError},
+    conversion::{
+        api::{Api, SubclassName},
+        error_reporter::convert_item_apis,
+        ConvertError,
+    },
     types::{validate_ident_ok_for_cxx, QualifiedName},
 };
 
@@ -43,18 +47,35 @@ pub(crate) fn check_names(apis: Vec<Api<FnAnalysis>>) -> Vec<Api<FnAnalysis>> {
             if let Some(ref cpp_name) = name.cpp_name {
                 // The C++ name might itself be outer_type::inner_type and thus may
                 // have multiple segments.
-                validate_all_segments_ok_for_cxx(QualifiedName::new_from_cpp_name(cpp_name).segment_iter())?;
+                validate_all_segments_ok_for_cxx(
+                    QualifiedName::new_from_cpp_name(cpp_name).segment_iter(),
+                )?;
             }
-            Ok(Some(api))
+            Ok(Box::new(std::iter::once(api)))
         }
-        Api::Function { .. } // we don't handle functions here because
+        Api::Subclass {
+            name: SubclassName(ref name),
+            ref superclass,
+        } => {
+            validate_all_segments_ok_for_cxx(name.name.segment_iter())?;
+            validate_all_segments_ok_for_cxx(superclass.segment_iter())?;
+            Ok(Box::new(std::iter::once(api)))
+        }
+        Api::Function { ref name, .. } => {
+            // we don't handle function names here because
             // the function analysis does an equivalent check. Instead of just rejecting
             // the function, it creates a wrapper function instead with a more
             // palatable name. That's preferable to rejecting the API entirely.
-        | Api::ConcreteType  { .. }
+            validate_all_segments_ok_for_cxx(name.name.segment_iter())?;
+            Ok(Box::new(std::iter::once(api)))
+        }
+        Api::ConcreteType { .. }
         | Api::CType { .. }
         | Api::StringConstructor { .. }
-        | Api::IgnoredItem { .. } => Ok(Some(api)),
+        | Api::RustType { .. }
+        | Api::RustSubclassFn { .. }
+        | Api::RustSubclassConstructor { .. }
+        | Api::IgnoredItem { .. } => Ok(Box::new(std::iter::once(api))),
     });
 
     // Reject any names which are duplicates within the cxx bridge mod,
@@ -74,10 +95,10 @@ pub(crate) fn check_names(apis: Vec<Api<FnAnalysis>>) -> Vec<Api<FnAnalysis>> {
             if *names_found.entry(name).or_default() > 1usize {
                 Err(ConvertError::DuplicateCxxBridgeName)
             } else {
-                Ok(Some(api))
+                Ok(Box::new(std::iter::once(api)))
             }
         } else {
-            Ok(Some(api))
+            Ok(Box::new(std::iter::once(api)))
         }
     });
     results
