@@ -18,7 +18,7 @@ use crate::{
         codegen_cpp::type_to_cpp::type_to_cpp,
         ConvertError,
     },
-    known_types::known_types,
+    known_types::{known_types, CxxGenericType},
     types::{make_ident, Namespace, QualifiedName},
 };
 use autocxx_parser::IncludeCppConfig;
@@ -242,7 +242,7 @@ impl<'a> TypeConverter<'a> {
         let (typ, tn) = match self.resolve_typedef(&original_tn) {
             None => (typ, original_tn),
             Some(Type::Path(resolved_tp)) => {
-                let resolved_tn = QualifiedName::from_type_path(&resolved_tp);
+                let resolved_tn = QualifiedName::from_type_path(resolved_tp);
                 deps.insert(resolved_tn.clone());
                 (resolved_tp.clone(), resolved_tn)
             }
@@ -276,13 +276,17 @@ impl<'a> TypeConverter<'a> {
 
         // Finally let's see if it's generic.
         if let Some(last_seg) = Self::get_generic_args(&mut typ) {
-            if known_types().is_cxx_acceptable_generic(&tn) {
+            let generic_behavior = known_types().cxx_generic_behavior(&tn);
+            let forward_declarations_ok = generic_behavior == CxxGenericType::Rust
+                || ctx.allow_instantiation_of_forward_declaration();
+            if generic_behavior != CxxGenericType::Not {
                 // this is a type of generic understood by cxx (e.g. CxxVector)
                 // so let's convert any generic type arguments. This recurses.
                 self.confirm_inner_type_is_acceptable_generic_payload(
                     &last_seg.arguments,
                     &tn,
-                    ctx,
+                    generic_behavior,
+                    forward_declarations_ok,
                 )?;
                 if let PathArguments::AngleBracketed(ref mut ab) = last_seg.arguments {
                     let mut innerty = self.convert_punctuated(ab.args.clone(), ns)?;
@@ -327,7 +331,7 @@ impl<'a> TypeConverter<'a> {
                     let mut innerty =
                         self.convert_type(t, ns, &TypeConversionContext::CxxInnerType)?;
                     types_encountered.extend(innerty.types_encountered.drain());
-                    extra_apis.extend(innerty.extra_apis.drain(..));
+                    extra_apis.append(&mut innerty.extra_apis);
                     GenericArgument::Type(innerty.ty)
                 }
                 _ => arg,
@@ -342,7 +346,7 @@ impl<'a> TypeConverter<'a> {
     }
 
     fn resolve_typedef<'b>(&'b self, tn: &QualifiedName) -> Option<&'b Type> {
-        self.typedefs.get(&tn).map(|resolution| match resolution {
+        self.typedefs.get(tn).map(|resolution| match resolution {
             Type::Path(typ) => {
                 let tn = QualifiedName::from_type_path(typ);
                 self.resolve_typedef(&tn).unwrap_or(resolution)
@@ -404,7 +408,8 @@ impl<'a> TypeConverter<'a> {
         &self,
         path_args: &PathArguments,
         desc: &QualifiedName,
-        ctx: &TypeConversionContext,
+        generic_behavior: CxxGenericType,
+        forward_declarations_ok: bool,
     ) -> Result<(), ConvertError> {
         // For now, all supported generics accept the same payloads. This
         // may change in future in which case we'll need to accept more arguments here.
@@ -417,19 +422,28 @@ impl<'a> TypeConverter<'a> {
                 for inner in &ab.args {
                     match inner {
                         GenericArgument::Type(Type::Path(typ)) => {
-                            let inner_qn = QualifiedName::from_type_path(&typ);
-                            if !ctx.allow_instantiation_of_forward_declaration()
+                            let inner_qn = QualifiedName::from_type_path(typ);
+                            if !forward_declarations_ok
                                 && self.forward_declarations.contains(&inner_qn)
                             {
                                 return Err(ConvertError::TypeContainingForwardDeclaration(
                                     inner_qn,
                                 ));
                             }
+                            if generic_behavior == CxxGenericType::Rust {
+                                if !inner_qn.get_namespace().is_empty() {
+                                    return Err(ConvertError::RustTypeWithAPath(inner_qn));
+                                }
+                                if !self.config.is_rust_type(&inner_qn.get_final_ident()) {
+                                    return Err(ConvertError::BoxContainingNonRustType(inner_qn));
+                                }
+                            }
                             if let Some(more_generics) = typ.path.segments.last() {
                                 self.confirm_inner_type_is_acceptable_generic_payload(
                                     &more_generics.arguments,
                                     desc,
-                                    ctx,
+                                    generic_behavior,
+                                    forward_declarations_ok,
                                 )?;
                             }
                         }
@@ -452,11 +466,15 @@ impl<'a> TypeConverter<'a> {
                 | Api::ConcreteType { .. }
                 | Api::Typedef { .. }
                 | Api::Enum { .. }
-                | Api::Struct { .. } => Some(api.name()),
+                | Api::Struct { .. }
+                | Api::Subclass { .. }
+                | Api::RustType { .. } => Some(api.name()),
                 Api::StringConstructor { .. }
                 | Api::Function { .. }
                 | Api::Const { .. }
                 | Api::CType { .. }
+                | Api::RustSubclassFn { .. }
+                | Api::RustSubclassConstructor { .. }
                 | Api::IgnoredItem { .. } => None,
             })
             .cloned()
