@@ -15,8 +15,11 @@
 use proc_macro2::TokenStream;
 use quote::quote;
 use syn::{
-    parse::Parser, parse_quote, punctuated::Punctuated, token::Unsafe, Attribute, FnArg,
-    ForeignItem, Ident, ImplItem, Item, ReturnType,
+    parse::Parser,
+    parse_quote,
+    punctuated::Punctuated,
+    token::{Comma, Unsafe},
+    Attribute, FnArg, ForeignItem, Ident, ImplItem, Item, ReturnType,
 };
 
 use super::{
@@ -138,20 +141,21 @@ pub(super) fn gen_function(
         #vis #unsafety fn #cxxbridge_name ( #params ) #ret_type;
     ));
     RsCodegenResult {
-        extern_c_mod_item: Some(extern_c_mod_item),
+        extern_c_mod_items: vec![extern_c_mod_item],
         bridge_items: Vec::new(),
         global_items: Vec::new(),
-        bindgen_mod_item: None,
+        bindgen_mod_items: Vec::new(),
         impl_entry,
         materialization,
+        extern_rust_mod_items: Vec::new(),
     }
 }
 
 fn generate_arg_lists(
     param_details: &[ArgumentAnalysis],
     is_constructor: bool,
-) -> (Punctuated<FnArg, syn::Token![,]>, Vec<TokenStream>) {
-    let mut wrapper_params: Punctuated<FnArg, syn::Token![,]> = Punctuated::new();
+) -> (Punctuated<FnArg, Comma>, Vec<TokenStream>) {
+    let mut wrapper_params: Punctuated<FnArg, Comma> = Punctuated::new();
     let mut arg_list = Vec::new();
 
     for pd in param_details {
