@@ -16,7 +16,7 @@ use std::collections::HashSet;
 
 use crate::{
     conversion::{
-        api::{ApiName, TypedefKind, UnanalyzedApi},
+        api::{Api, ApiName, SubclassName, TypedefKind, UnanalyzedApi},
         ConvertError,
     },
     types::Namespace,
@@ -102,14 +102,24 @@ impl<'a> ParseBindgen<'a> {
     ) -> Result<Vec<UnanalyzedApi>, ConvertError> {
         let items = Self::find_items_in_root(items)?;
         if !self.config.exclude_utilities() {
-            generate_utilities(&mut self.apis, &self.config);
+            generate_utilities(&mut self.apis, self.config);
         }
+        self.add_subclasses();
         let root_ns = Namespace::new();
         self.parse_mod_items(items, root_ns);
         self.confirm_all_generate_directives_obeyed()?;
         Ok(self.apis)
     }
 
+    fn add_subclasses(&mut self) {
+        self.apis
+            .extend(self.config.subclasses.iter().map(|sc| Api::Subclass {
+                // TODO support namespaces
+                name: SubclassName::new(sc.subclass.clone()),
+                superclass: QualifiedName::new_from_cpp_name(&sc.superclass),
+            }));
+    }
+
     fn find_items_in_root(items: Vec<Item>) -> Result<Vec<Item>, ConvertError> {
         for item in items {
             match item {
@@ -164,7 +174,9 @@ impl<'a> ParseBindgen<'a> {
                 // cxx::bridge can't cope with type aliases to generic
                 // types at the moment.
                 let name = api_name_qualified(ns, s.ident.clone(), &s.attrs)?;
-                let api = if is_forward_declaration {
+                let api = if ns.is_empty() && self.config.is_rust_type(&s.ident) {
+                    UnanalyzedApi::RustType { name }
+                } else if is_forward_declaration {
                     UnanalyzedApi::ForwardDeclaration { name }
                 } else {
                     UnanalyzedApi::Struct {
