@@ -33,6 +33,7 @@ enum Behavior {
     CByValue,
     CVariableLengthByValue,
     CVoid,
+    RustContainerByValueSafe,
 }
 
 /// Details about known special types, mostly primitives.
@@ -70,13 +71,14 @@ impl TypeDetails {
             | Behavior::RustStr
             | Behavior::CxxString
             | Behavior::CxxContainerByValueSafe
-            | Behavior::CxxContainerNotByValueSafe => {
+            | Behavior::CxxContainerNotByValueSafe
+            | Behavior::RustContainerByValueSafe => {
                 let tn = QualifiedName::new_from_cpp_name(&self.rs_name);
                 let cxx_name = tn.get_final_item();
                 let (templating, payload) = match self.behavior {
-                    Behavior::CxxContainerByValueSafe | Behavior::CxxContainerNotByValueSafe => {
-                        ("template<typename T> ", "T* ptr")
-                    }
+                    Behavior::CxxContainerByValueSafe
+                    | Behavior::CxxContainerNotByValueSafe
+                    | Behavior::RustContainerByValueSafe => ("template<typename T> ", "T* ptr"),
                     _ => ("", "char* ptr"),
                 };
                 Some(format!(
@@ -115,6 +117,16 @@ impl TypeDetails {
     fn to_typename(&self) -> QualifiedName {
         QualifiedName::new_from_cpp_name(&self.rs_name)
     }
+
+    fn get_generic_behavior(&self) -> CxxGenericType {
+        match self.behavior {
+            Behavior::CxxContainerByValueSafe | Behavior::CxxContainerNotByValueSafe => {
+                CxxGenericType::Cpp
+            }
+            Behavior::RustContainerByValueSafe => CxxGenericType::Rust,
+            _ => CxxGenericType::Not,
+        }
+    }
 }
 
 /// Database of known types.
@@ -130,6 +142,18 @@ pub(crate) fn known_types() -> &'static TypeDatabase {
     KNOWN_TYPES.get_or_init(create_type_database)
 }
 
+/// The type of payload that a cxx generic can contain.
+#[derive(PartialEq, Clone, Copy)]
+pub enum CxxGenericType {
+    /// Not a generic at all
+    Not,
+    /// Some generic like cxx::UniquePtr where the contents must be a
+    /// complete type.
+    Cpp,
+    /// Some generic like rust::Box where forward declarations are OK
+    Rust,
+}
+
 impl TypeDatabase {
     fn get(&self, ty: &QualifiedName) -> Option<&TypeDetails> {
         // The following line is important. It says that
@@ -169,7 +193,8 @@ impl TypeDatabase {
                     | Behavior::RustString
                     | Behavior::RustByValue
                     | Behavior::CByValue
-                    | Behavior::CVariableLengthByValue => true,
+                    | Behavior::CVariableLengthByValue
+                    | Behavior::RustContainerByValueSafe => true,
                     Behavior::CxxString
                     | Behavior::CxxContainerNotByValueSafe
                     | Behavior::CVoid => false,
@@ -182,7 +207,7 @@ impl TypeDatabase {
     /// but a reference in Rust. This only applies to rust::Str
     /// (C++ name) which is &str in Rust.
     pub(crate) fn should_dereference_in_cpp(&self, tn: &QualifiedName) -> bool {
-        self.get(&tn)
+        self.get(tn)
             .map(|td| matches!(td.behavior, Behavior::RustStr))
             .unwrap_or(false)
     }
@@ -193,7 +218,7 @@ impl TypeDatabase {
     /// any PathArguments within this TypePath - callers should
     /// put them back again if needs be.
     pub(crate) fn consider_substitution(&self, tn: &QualifiedName) -> Option<TypePath> {
-        self.get(&tn).map(|td| td.to_type_path())
+        self.get(tn).map(|td| td.to_type_path())
     }
 
     pub(crate) fn special_cpp_name(&self, rs: &QualifiedName) -> Option<String> {
@@ -232,15 +257,10 @@ impl TypeDatabase {
     /// Whether this is a generic type acceptable to cxx. Otherwise,
     /// if we encounter a generic, we'll replace it with a synthesized concrete
     /// type.
-    pub(crate) fn is_cxx_acceptable_generic(&self, ty: &QualifiedName) -> bool {
+    pub(crate) fn cxx_generic_behavior(&self, ty: &QualifiedName) -> CxxGenericType {
         self.get(ty)
-            .map(|x| {
-                matches!(
-                    x.behavior,
-                    Behavior::CxxContainerByValueSafe | Behavior::CxxContainerNotByValueSafe
-                )
-            })
-            .unwrap_or(false)
+            .map(|x| x.get_generic_behavior())
+            .unwrap_or(CxxGenericType::Not)
     }
 
     pub(crate) fn is_cxx_acceptable_receiver(&self, ty: &QualifiedName) -> bool {
@@ -318,6 +338,12 @@ fn create_type_database() -> TypeDatabase {
         Behavior::RustString,
         None,
     ));
+    db.insert(TypeDetails::new(
+        "std::boxed::Box",
+        "rust::Box",
+        Behavior::RustContainerByValueSafe,
+        None,
+    ));
     db.insert(TypeDetails::new(
         "i8",
         "int8_t",
@@ -402,6 +428,7 @@ pub(crate) fn type_lacks_copy_constructor(ty: &Type) -> bool {
         Type::Path(typ) => {
             let tn = QualifiedName::from_type_path(typ);
             tn.to_cpp_name().starts_with("std::unique_ptr")
+                || tn.to_cpp_name().starts_with("rust::Box")
         }
         _ => false,
     }
