@@ -82,8 +82,8 @@ pub use cxx;
 pub struct CppFilePair {
     /// Declarations to go into a header file.
     pub header: Vec<u8>,
-    /// Implementations to go into a .cpp file, if any.
-    pub implementation: Option<Vec<u8>>,
+    /// Implementations to go into a .cpp file.
+    pub implementation: Vec<u8>,
     /// The name which should be used for the header file
     /// (important as it may be `#include`d elsewhere)
     pub header_name: String,
@@ -372,12 +372,12 @@ impl IncludeCppEngine {
         }
 
         let mod_name = self.config.get_mod_name();
-        let mut builder = self.make_bindgen_builder(&inc_dirs, &extra_clang_args);
+        let mut builder = self.make_bindgen_builder(&inc_dirs, extra_clang_args);
         if let Some(dep_recorder) = dep_recorder {
             builder = builder.parse_callbacks(Box::new(AutocxxParseCallbacks(dep_recorder)));
         }
         let header_contents = self.build_header();
-        self.dump_header_if_so_configured(&header_contents, &inc_dirs, &extra_clang_args);
+        self.dump_header_if_so_configured(&header_contents, &inc_dirs, extra_clang_args);
         let header_and_prelude = format!("{}\n\n{}", known_types().get_prelude(), header_contents);
         builder = builder.header_contents("example.hpp", &header_and_prelude);
 
@@ -481,7 +481,7 @@ pub fn do_cxx_cpp_generation(rs: TokenStream2) -> Result<CppFilePair, cxx_gen::E
     Ok(CppFilePair {
         header: cxx_generated.header,
         header_name: "cxxgen.h".into(),
-        implementation: Some(cxx_generated.implementation),
+        implementation: cxx_generated.implementation,
     })
 }
 
